namespace NetinaShop.AdminPanel.PWA.Services.RestServices;

public interface IUserRestApi
{
    [Put("")]
    Task UpdateUserAsync([Body]UserActionRequestDto request, [Header("Authorization")] string authorization);
    [Post("")]
    Task CreateUserAsync([Body] UserActionRequestDto request, [Header("Authorization")] string authorization);

    [Get("/{id}")]
    Task<ApplicationUserSDto> ReadOne(Guid id, [Header("Authorization")] string authorization);
    [Get("")]
    Task<List<ApplicationUserSDto>> ReadAll([Query] int page, [Header("Authorization")] string authorization);
    [Get("")]
    Task<List<ApplicationUserSDto>> ReadAll([Query] int page, [Query] string userPhone, [Header("Authorization")] string authorization);
}